/*
 * Copyright (C) - 2012 David Goulet <dgoulet@efficios.com>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */

#include <assert.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdbool.h>

#define TRACEPOINT_DEFINE
#include "tp.h"

void create_file(const char *path)
{
	int ret;

	assert(path);

	ret = creat(path, S_IRWXU);
	if (ret < 0) {
		fprintf(stderr, "Failed to create file %s\n", path);
		return;
	}

	(void) close(ret);
}

int main(int argc, char **argv)
{
	unsigned int i, netint;
	long values[] = { 1, 2, 3 };
	char text[10] = "test";
	double dbl = 2.0;
	float flt = 2222.0;
	int nr_iter = 100;
	useconds_t nr_usec = 0;
	char *tmp_file_path = NULL;
	bool file_created = false;

	if (argc >= 2) {
		/*
		 * If nr_iter is negative, do an infinite tracing loop.
		 */
		nr_iter = atoi(argv[1]);
	}

	if (argc >= 3) {
		/* By default, don't wait unless user specifies. */
		nr_usec = atoi(argv[2]);
	}

	if (argc >= 4) {
		tmp_file_path = argv[3];
	}

	for (i = 0; nr_iter < 0 || i < nr_iter; i++) {
		netint = htonl(i);
		tracepoint(tp, tptest, i, netint, values, text, strlen(text), dbl,
				flt);

		/*
		 * First loop we create the file if asked to indicate that at least one
		 * tracepoint has been hit.
		 */
		if (!file_created && tmp_file_path) {
			create_file(tmp_file_path);
			file_created = true;
		}
		usleep(nr_usec);
	}

	return 0;
}
